@@ -64,7 +64,8 @@ tooltip <- function(text, placement = "top") {
     `data-toggle` = "tooltip",
     title = text,
     icon("question-sign", lib = "glyphicon"),
-    `data-placement` = placement
+    `data-placement` = placement,
+    `data-html` = "true"
   )
 }
 
@@ -167,14 +168,14 @@ codeGenerators <- list(
 
       # Get unescaped filenames in a char vector, with full path
       filepaths <- vapply(event$value, `[[`, "name", FUN.VALUE = "")
-      filepaths <- file.path(app$getAppDir(), "tests", filepaths)
+      filepaths <- file.path(app$getTestsDir(), filepaths)
 
       # Check that all files exist. If not, add a message and don't run test
       # automatically on exit.
       if (!all(file.exists(filepaths))) {
-        add_dont_run_reason("An uploadFile() must be updated: use the correct path relative to the app's tests/ directory, or copy the file to the app's tests/ directory.")
+        add_dont_run_reason("An uploadFile() must be updated: use the correct path relative to the app's tests/shinytest directory, or copy the file to the app's tests/shinytest directory.")
         code <- paste0(code,
-          " # <-- This should be the path to the file, relative to the app's tests/ directory"
+          " # <-- This should be the path to the file, relative to the app's tests/shinytest directory"
         )
       }
 
@@ -283,9 +284,9 @@ generateTestCode <- function(events, name, seed, useTimes = FALSE,
     if (load_mode) {
       'app <- ShinyLoadDriver$new()'
     } else {
-
       paste0(
-        'app <- ShinyDriver$new("', paste("..", app$getAppFilename(), sep = "/"), '"',
+        # Need paste instead of file.path because app$getAppFileName() can be NULL which makes file.path grumpy.
+        'app <- ShinyDriver$new("', paste(app$getRelativePathToApp(), app$getAppFilename(), sep="/"), '"',
         if (!is.null(seed)) sprintf(", seed = %s", seed),
         if (!is.null(load_timeout)) paste0(", loadTimeout = ", load_timeout),
         if (length(shiny_options) > 0) paste0(", shinyOptions = ", deparse2(shiny_options)),
@@ -331,7 +332,7 @@ shinyApp(
               style = "display: inline;"
             ),
             tooltip(
-              "You can also Ctrl-click or ⌘-click on an output to snapshot just that one output.\n To trigger a snapshot via the keyboard, press Ctrl-shift-S or ⌘-shift-S",
+              HTML("You can also Ctrl-click or &#8984;-click on an output to snapshot just that one output.<br> To trigger a snapshot via the keyboard, press Ctrl-shift-S or &#8984;-shift-S"),
               placement = "bottom"
             ),
             hr()
@@ -408,7 +409,7 @@ shinyApp(
     }
 
     saveFile <- reactive({
-      file.path(app$getAppDir(), "tests", paste0(input$testname, ".R"))
+      file.path(app$getTestsDir(), paste0(input$testname, ".R"))
     })
 
     # Number of snapshot or fileDownload events in input$testevents
